#!/bin/sh

#--------------------

#param="Mth=3.0 Mz=3.0"
param="$*"

echo "ETASX.sh $param" > etas.run

# VERBOSE を標準で 0 -> 1 にした 2015/05/21
#-VERBOSE=0
VERBOSE=1
while getopts ABVrhNQ123 OPT
do
  case $OPT in
    "A" ) XAFTPOI="TRUE" ;;
    "B" ) FINDBEST="TRUE" ;;
    "V" ) VERBOSE=1 ;;
    "r" ) RESIDUAL="TRUE" ;;
    "h" ) USAGE="TRUE" ;;
    "N" ) NOLASTLOOP="TRUE" ;;
    "Q" ) QUICK="TRUE" ;;
    "1" ) FIRST="TRUE" ;;
    "2" ) LATTER="TRUE" ;;
    "3" ) ALL="TRUE" ;;
  esac
done

if [ "$USAGE" = "TRUE" ]; then
  cat <<EOM
Usage ETASX.sh OPTION ARGS
     OPTION -A use aftpoi prog
            -B search BEST solution
            -V verbose
            -r residual plot
            -N do not last loop
            -Q initial parameter
            -h print HELP
     ARGVs  K=K
            c=c
            alpha=alpha
            (mu=mu)
            (p=p)
            Mth=Mth
            Mz=Mz
            zts=zts
            tstart=tstart
            zte=zte
            tend=tend
            fix=fixparm
EOM
  \rm etas.lock
  exit 0
fi

echo "==================================== ETASX ===========================================" 
echo "==================================== ETASX ===========================================" >> work.log

if [ -f etas.lock ]; then
	echo "ETAS is running"
        echo "exit"
	exit
else
	echo "touch etas.lock"
	touch etas.lock
fi

###############################
#ETASDIR='/Users/tsuru/etas/bin'
ETASDIR=$ETAS_HOME/bin
###############################

if [ "$NOLASTLOOP" = "TRUE" ]; then
  param="$param NOLASTLOOP"
fi

if [ "$QUICK" = "TRUE" ]; then
  param="$param QUICK"
fi

if [ "$XAFTPOI" = "TRUE" ]; then
  param="$param alpha=0.0"
  PROG_ETAS="$ETASDIR/aftpoi"
  PROG_RETAS="$ETASDIR/raftpoi"
else
  PROG_ETAS="$ETASDIR/etas"
  PROG_RETAS="$ETASDIR/retas"
fi

PROG_ETASXPL="$ETASDIR/etasx.pl $PROG_ETAS $VERBOSE"
if [ "$RESIDUAL" = "TRUE" ]; then
  PROG_ETASPLOT="$ETASDIR/etasrplot"
else
  PROG_ETASPLOT="$ETASDIR/etasplot"
fi
PROG_MKETASOPENPL="$ETASDIR/mk_etasopen.pl"

#################################
# RUN start
#################################

if [ "$FINDBEST" = "TRUE" ]; then
  $PROG_MKETASOPENPL mu=0.0 p=1.2 $param > etas.open
  $PROG_ETASXPL
  \mv etas.open etas.open.1
  \mv etas.out etas.out.1

  $PROG_MKETASOPENPL mu=0.0 p=1.0 $param > etas.open
  $PROG_ETASXPL
  \mv etas.open etas.open.2
  \mv etas.out etas.out.2

  $PROG_MKETASOPENPL mu=0.1 p=1.2 $param > etas.open
  $PROG_ETASXPL
  \mv etas.open etas.open.3
  \mv etas.out etas.out.3

  $PROG_MKETASOPENPL mu=0.1 p=1.0 $param > etas.open
  $PROG_ETASXPL
  \mv etas.open etas.open.4
  \mv etas.out etas.out.4

  more etas.out.{1,2,3,4} > etas.all

  AIC1=`tail +5 etas.out.1 | head -1`
  AIC2=`tail +5 etas.out.2 | head -1`
  AIC3=`tail +5 etas.out.3 | head -1`
  AIC4=`tail +5 etas.out.4 | head -1`

  BEST=`$ETASDIR/BEST.pl $AIC1 $AIC2 $AIC3 $AIC4`

  \cp etas.open.$BEST etas.open
  \cp etas.out.$BEST etas.out
else
#  $PROG_MKETASOPENPL mu=0.0 p=1.2 $param > etas.open
  $PROG_MKETASOPENPL $param > etas.open
  $PROG_ETASXPL $param >> work.log 2>&1
fi

#-$PROG_RETAS >> work.log 2>&1
$PROG_RETAS
#$PROG_ETASPLOT &
#$PROG_ETASPLOT 10

if [ "$FINDBEST" = "TRUE" ]; then
  cat etas.all
  echo "********************************************"
  echo  $AIC1 $AIC2 $AIC3 $AIC4
  echo "BEST=$BEST"
  echo "  1: mu=0  p<>1"
  echo "  2: mu=0  p=1"
  echo "  3: mu<>0 p<>1"
  echo "  4: mu<>0 p=1"
  echo "********************************************"
else
  cat etas.out
  echo "### cal END ###"
  echo "*** cal END ***" >> work.log 2>&1
fi

\rm etas.lock
