#!/bin/sh

# for EIC
#export ETAS_HOME=/usr/local/prog/etas

if [ "$ETAS_OPTION" = "" ]; then
 export ETAS_OPTION=""
fi

#awk 'if ($5>=$CM1 && $5 <=$CM2) {printf("%f",$5+0.0001);}' work.etas | grep -v days > t.xy
x0=50.0
y0=20.0

param="$*"
echo "etascm.sh $param" > etascm.run

while getopts hs:e:i:z:t:r:m:K:c:a:p:f:CS:E:D:WXPO OPT
do
  case $OPT in
    "h" ) HELP="TRUE" ;;
    "s" ) tstart="$OPTARG" ;;
    "e" ) tend="$OPTARG" ;;
    "i" ) zts="$OPTARG" ;;
    "z" ) zte="$OPTARG" ;;
    "t" ) Mth="$OPTARG" ;;
    "r" ) Mz="$OPTARG" ;;
    "m" ) mu="$OPTARG" ;;
    "K" ) K="$OPTARG" ;;
    "c" ) c="$OPTARG" ;;
    "a" ) alpha="$OPTARG" ;;
    "p" ) p="$OPTARG" ;;
    "f" ) fix="$OPTARG" ;;
    "C" ) CONSOLE="TRUE" ;;
    "S" ) CM1="$OPTARG" ;;
    "E" ) CM2="$OPTARG" ;;
    "D" ) DT="$OPTARG" ;;
    "W" ) WORKETAS="TRUE" ;;
    "X" ) PLOTX="TRUE" ;;
    "P" ) OUTPUT="TRUE" ;;
  esac
done

if [ "$HELP" = "TRUE" ]; then
  cat <<EOM
Usage etascm.sh OPTION
     OPTION -h help
            -s tstart
            -e tend
            -i zts
            -z zte
            -t Mth
            -r Mz
            -m mu
            -K K
            -c c
            -a alpha
            -p p
            -f fix
            -C CONSOLE mode
            -X Plot Xwindow
            -P output PS
e.g. etascm.sh -i0.0 -z200.0 -s0.01 -e100.0 -t5.0 -r9.0 -m0.01 -K10.0 -c0.03 -a1.5 -p1.2 -f00100 -S1.0 -E10.0 -D1.0
     etascm.sh -i0.0 -z25.676926 -s0.01 -e25.676926 -t3.0 -r7.3 -m0.101605 -K33.4685 -c0.00981207 -a1.82649 -p1.096 -f00100 -S1.0 -E5.0 -D0.5
EOM
  exit 0
fi

if [ "$WORKETAS" = "TRUE" ]; then
  CMs=`awk -v "cm1=$CM1" -v "cm2=$CM2" -v "mth=$Mth" '{if ($5 >= cm1 && $5 <= cm2 && $4 >= mth) {printf("%f\n",$5+0.001);}}' work.etas | grep -v days`
else
  CMs=`seq $CM1 $DT $CM2`;
fi

cat /dev/null > etascm.dat
cat /dev/null > work.log
#cat /dev/null > CMparams.log

#echo "$CMs" > CM.dat

for CM in $CMs; do
echo "============ $CM ==========="
AIC=""
x0=`echo $x0+10.0 | bc -l`
y0=`echo $y0+5.0 | bc -l`
$ETAS_HOME/bin/ETASX.sh -Q Mth=$Mth Mz=$Mz tstart=$tstart tend=$tend zts=$zts zte=$CM mu=$mu K=$K c=$c alpha=$alpha p=$p fix=$fix
zmu=`head -4 etas.out | tail -1 | awk '{printf("%g",$1);}'`
zK=`head -4 etas.out | tail -1 | awk '{printf("%g",$2);}'`
zc=`head -4 etas.out | tail -1 | awk '{printf("%g",$3);}'`
zalpha=`head -4 etas.out | tail -1 | awk '{printf("%g",$4);}'`
zp=`head -4 etas.out | tail -1 | awk '{printf("%g",$5);}'`
zaic2=`head -6 etas.out | tail -1 | awk '{printf("%g",$1);}'`
zgrad=`head -8 etas.out | tail -1 | awk '{printf("%g",$1);}'`
echo "mu= $zmu"
echo "K= $zK"
echo "c= $zc"
echo "alpha= $zalpha"
echo "p= $zp"
echo "aic/2= $zaic2"
echo "grad= $zgrad"
AIC=$zaic2
ETASparams="mu= $zmu K= $zK c= $zc alpha= $zalpha p= $zp aic= $AIC grad= $zgrad"
if [ "$PLOTX" = "TRUE" ]; then
  $ETAS_HOME/bin/etasplot $ETAS_OPTION -x$x0 -y$y0 &
fi
if [ "$OUTPUT" = "TRUE" ]; then
  echo "$ETAS_HOME/bin/etasplot -w10 -ocm_CM${CM}"
  $ETAS_HOME/bin/etasplot -w10 -ocm_CM${CM}.ps
fi
echo "CM= $CM $ETASparams" >> CMparams.log
echo "$CM $zmu $zK $zc $zalpha $zp $zaic2 $zgrad" >> etascm.dat
done

echo "*** cat etascm.dat ***"
\cat etascm.dat
echo "*** cal ETASCM END ***"
