#!/bin/sh

# for EIC
#export ETAS_HOME=/usr/local/prog/etas

if [ "$ETAS_OPTION" = "" ]; then
 export ETAS_OPTION=""
fi

#awk 'if ($5>=$CT1 && $5 <=$CT2) {printf("%f",$5+0.0001);}' work.etas | grep -v days > t.xy
x0=50.0
y0=20.0

param="$*"
echo "etasct.sh $param" > etasct.run

while getopts hs:e:i:z:t:r:m:K:c:a:p:f:CS:E:D:WXPO OPT
do
  case $OPT in
    "h" ) HELP="TRUE" ;;
    "s" ) tstart="$OPTARG" ;;
    "e" ) tend="$OPTARG" ;;
    "i" ) zts="$OPTARG" ;;
    "z" ) zte="$OPTARG" ;;
    "t" ) Mth="$OPTARG" ;;
    "r" ) Mz="$OPTARG" ;;
    "m" ) mu="$OPTARG" ;;
    "K" ) K="$OPTARG" ;;
    "c" ) c="$OPTARG" ;;
    "a" ) alpha="$OPTARG" ;;
    "p" ) p="$OPTARG" ;;
    "f" ) fix="$OPTARG" ;;
    "C" ) CONSOLE="TRUE" ;;
    "S" ) CT1="$OPTARG" ;;
    "E" ) CT2="$OPTARG" ;;
    "D" ) DT="$OPTARG" ;;
    "W" ) WORKETAS="TRUE" ;;
    "X" ) PLOTX="TRUE" ;;
    "P" ) OUTPUT="TRUE" ;;
  esac
done

if [ "$HELP" = "TRUE" ]; then
  cat <<EOM
Usage etasct.sh OPTION
     OPTION -h help
            -s tstart
            -e tend
            -i zts
            -z zte
            -t Mth
            -r Mz
            -m mu
            -K K
            -c c
            -a alpha
            -p p
            -f fix
            -C CONSOLE mode
            -X Plot Xwindow
            -P output PS
e.g. etasct.sh -i0.0 -z200.0 -s0.01 -e100.0 -t5.0 -r9.0 -m0.01 -K10.0 -c0.03 -a1.5 -p1.2 -f00100 -S1.0 -E10.0 -D1.0
     etasct.sh -i0.0 -z25.676926 -s0.01 -e25.676926 -t3.0 -r7.3 -m0.101605 -K33.4685 -c0.00981207 -a1.82649 -p1.096 -f00100 -S1.0 -E5.0 -D0.5
EOM
  exit 0
fi

if [ "$WORKETAS" = "TRUE" ]; then
  CTs=`awk -v "ct1=$CT1" -v "ct2=$CT2" -v "mth=$Mth" '{if ($5 >= ct1 && $5 <= ct2 && $4 >= mth) {printf("%f\n",$5+0.001);}}' work.etas | grep -v days`
else
  CTs=`seq $CT1 $DT $CT2`;
fi

cat /dev/null > etasct.dat
cat /dev/null > work.log
#cat /dev/null > CTparams.log

#Mth=3.000000 
#Mz=7.300000 
#tstart=0.010000 
#tend=25.676920 
#zts=0.000000 
#zte=25.676920 
#mu=0.101605     
#K=33.4685      
#c=0.00981207   
#alpha=1.82649      
#p=1.096        
#fix=00100

$ETAS_HOME/bin/ETASX.sh -Q Mth=$Mth Mz=$Mz tstart=$tstart tend=$tend zts=$zts zte=$zte mu=$mu K=$K c=$c alpha=$alpha p=$p fix=$fix
zmu=`head -4 etas.out | tail -1 | awk '{printf("%g",$1);}'`
zK=`head -4 etas.out | tail -1 | awk '{printf("%g",$2);}'`
zc=`head -4 etas.out | tail -1 | awk '{printf("%g",$3);}'`
zalpha=`head -4 etas.out | tail -1 | awk '{printf("%g",$4);}'`
zp=`head -4 etas.out | tail -1 | awk '{printf("%g",$5);}'`
zaic2=`head -6 etas.out | tail -1 | awk '{printf("%g",$1);}'`
echo "mu= $zmu"
echo "K= $zK"
echo "c= $zc"
echo "alpha= $zalpha"
echo "p= $zp"
echo "aic/2= $zaic2"
AIC0=$zaic2

if [ "$PLOTX" = "TRUE" ]; then
  $ETAS_HOME/bin/etasplot $ETAS_OPTION -x$x0 -y$y0 &
fi
if [ "$OUTPUT" = "TRUE" ]; then
  echo "$ETAS_HOME/bin/etasplot -w10 -octa0"
  $ETAS_HOME/bin/etasplot -w10 -octa0.ps
fi

#echo "$CTs" > CT.dat

for CT in $CTs; do
echo "============ $CT ==========="
AIC1=""
AIC2=""
x0=`echo $x0+10.0 | bc -l`
y0=`echo $y0+5.0 | bc -l`
$ETAS_HOME/bin/ETASX.sh -Q Mth=$Mth Mz=$Mz tstart=$tstart tend=$tend zts=$zts zte=$CT mu=$mu K=$K c=$c alpha=$alpha p=$p fix=$fix
zmu=`head -4 etas.out | tail -1 | awk '{printf("%g",$1);}'`
zK=`head -4 etas.out | tail -1 | awk '{printf("%g",$2);}'`
zc=`head -4 etas.out | tail -1 | awk '{printf("%g",$3);}'`
zalpha=`head -4 etas.out | tail -1 | awk '{printf("%g",$4);}'`
zp=`head -4 etas.out | tail -1 | awk '{printf("%g",$5);}'`
zaic2=`head -6 etas.out | tail -1 | awk '{printf("%g",$1);}'`
zgrad=`head -8 etas.out | tail -1 | awk '{printf("%g",$1);}'`
echo "mu= $zmu"
echo "K= $zK"
echo "c= $zc"
echo "alpha= $zalpha"
echo "p= $zp"
echo "aic/2= $zaic2"
echo "grad= $zgrad"
AIC1=$zaic2
ETASparams1="mu= $zmu K= $zK c= $zc alpha= $zalpha p= $zp aic= $AIC1 grad= $zgrad"
if [ "$PLOTX" = "TRUE" ]; then
  $ETAS_HOME/bin/etasplot $ETAS_OPTION -x$x0 -y$y0 &
fi
if [ "$OUTPUT" = "TRUE" ]; then
  echo "$ETAS_HOME/bin/etasplot -w10 -octa1_CT${CT}"
  $ETAS_HOME/bin/etasplot -w10 -octa1_CT${CT}.ps
fi

$ETAS_HOME/bin/ETASX.sh -Q Mth=$Mth Mz=$Mz tstart=$CT tend=$tend zts=$zts zte=$zte mu=$mu K=$K c=$c alpha=$alpha p=$p fix=$fix
zmu=`head -4 etas.out | tail -1 | awk '{printf("%g",$1);}'`
zK=`head -4 etas.out | tail -1 | awk '{printf("%g",$2);}'`
zc=`head -4 etas.out | tail -1 | awk '{printf("%g",$3);}'`
zalpha=`head -4 etas.out | tail -1 | awk '{printf("%g",$4);}'`
zp=`head -4 etas.out | tail -1 | awk '{printf("%g",$5);}'`
zaic2=`head -6 etas.out | tail -1 | awk '{printf("%g",$1);}'`
zgrad=`head -8 etas.out | tail -1 | awk '{printf("%g",$1);}'`
echo "mu= $zmu"
echo "K= $zK"
echo "c= $zc"
echo "alpha= $zalpha"
echo "p= $zp"
echo "aic/2= $zaic2"
echo "grad= $zgrad"
AIC2=$zaic2
ETASparams2="mu= $zmu K= $zK c= $zc alpha= $zalpha p= $zp aic= $AIC2 grad= $zgrad"
x0=`echo $x0+10.0 | bc -l`
if [ "$PLOTX" = "TRUE" ]; then
  $ETAS_HOME/bin/etasplot $ETAS_OPTION -x$x0 -y$y0 &
fi
if [ "$OUTPUT" = "TRUE" ]; then
  echo "$ETAS_HOME/bin/etasplot -w10 -octa2_CT${CT}"
  $ETAS_HOME/bin/etasplot -w10 -octa2_CT${CT}.ps
fi

DAIC=`echo "$AIC1 + $AIC2 - $AIC0" | bc -l`
echo "CT= $CT DAIC= $DAIC"
echo "$CT $DAIC" >> etasct.dat
echo "CT= $CT $ETASparams1  $ETASparams2" >> CTparams.log
echo "CT= $CT dAIC= $DAIC" >> CTparams.log
done

echo "*** cat etasct.dat ***"
\cat etasct.dat
echo "*** cal ETASCT END ***"
