#!/bin/sh

# for EIC
#export ETAS_HOME=/usr/local/prog/etas

bvalue=0.9
CASE=0
tstart=0.01
nd=1000
MM=`date '+%M'`
SS=`date '+%S'`
NSEED=`echo "60*$SS + $MM" | bc -l`
MainShock=6.0

while getopts j:b:s:n:z:hft:r:m:K:c:a:p:CM: OPT
do
  case $OPT in
    "j" ) CASE="$OPTARG" ;;
    "b" ) bvalue="$OPTARG" ;;
    "s" ) tstart="$OPTARG" ;;
    "n" ) nd="$OPTARG" ;;
    "z" ) NSEED="TRUE" ;;
    "h" ) HELP="TRUE" ;;
    "f" ) FILE="TRUE" ;;
    "t" ) Mc="$OPTARG" ;;
    "r" ) Mz="$OPTARG" ;;
    "m" ) mu="$OPTARG" ;;
    "K" ) K="$OPTARG" ;;
    "c" ) c="$OPTARG" ;;
    "a" ) alpha="$OPTARG" ;;
    "p" ) p="$OPTARG" ;;
    "C" ) CONSOLE="TRUE" ;;
    "M" ) MainShock="$OPTARG" ;;
  esac
done

if [ "$HELP" = "TRUE" ]; then
  cat <<EOM
Usage etasim.sh OPTION
     OPTION -j CASE 0:GR 1:work.etas 2:bootstrap
            -b bvalue
            -s tstart
            -n nd simulation
            -z nseed
            -h help
            -f parameters from etas.out
            -t Mc
            -r Mz
            -m mu
            -K K
            -c c
            -a alpha
            -p p
            -C CONSOLE mode
e.g. etasim.sh -j0 -b0.9 -s0.01 -n1000 -f
     etasim.sh -j0 -b0.9 -s0.01 -n1000 -t5.0 -r9.0 -m0.01 -K10.0 -c0.03 -a1.5 -p1.2
EOM
  exit 0
fi

#if [ "$INITIAL" = "TRUE" ]; then
#  NSEED=0
#fi

if [ "$FILE" = "TRUE" ]; then
  MAGline=`head -3 etas.out | tail -1`
  ETASparam=`head -4 etas.out | tail -1`
else
  MAGline="$Mc $Mz" 
  ETASparam="$mu $K $c $alpha $p"
fi

echo "$CASE $bvalue"  > etasim.open
echo "$tstart $nd"   >> etasim.open
echo "$MAGline"      >> etasim.open
echo "$ETASparam"    >> etasim.open
echo "$MainShock"    >> etasim.open

$ETAS_HOME/bin/etasim etasim.open $NSEED > etasim.log 2>&1
$ETAS_HOME/bin/retasim > retasim.log 2>&1
if [ ! "$CONSOLE" = "TRUE" ]; then
  $ETAS_HOME/bin/etasimplot $ETAS_OPTION
fi
