/* psplot.c
   plot POSTSCRIPT file
   this program needs libpsl.a 
   Aug 14,1994 by H.Tsuruoka
   SEP  3, 1994 ADD psarc routine
   OCT 16, 1994 psimagefill routine
   JUN 14, 1994 PS_COLOR
   */
#include <stdio.h>
#include <memory.h>
#include <math.h>
#include <string.h>

struct EPS {			/* Holds info for eps files */
    int x0, x1, y0, y1;		/* Bounding box values in points */
    char *font[34];		/* Pointers to font names used */
    char *name;			/* User name */
    char *title;		/* Plot title */
};

static double scale;
static int shade;
static int PS_COLOR = 0;
static int PS_R = 255;
static int PS_G = 255;
static int PS_B = 255;
static int R, G, B;
static RGB[64][3] = { 0, 0, 0, 0, 0, 255, 0, 255, 0, 0, 255, 255, 255, 0, 0, 255, 0, 255, 255,
    255, 0, 255, 255, 255,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    255, 0, 0, 255, 32, 0, 255, 65, 0, 255, 98, 0, 255, 131, 0, 255, 164, 0,
    255, 197, 0, 255, 230, 0,
    246, 255, 0, 213, 255, 0, 180, 255, 0, 148, 255, 0, 115, 255, 0, 82, 255, 0,
    49, 255, 0, 16, 255, 0,
    0, 255, 16, 0, 255, 49, 0, 255, 82, 0, 255, 115, 0, 255, 148, 0, 255, 180,
    0, 255, 213, 0, 255, 246,
    0, 230, 255, 0, 197, 255, 0, 164, 255, 0, 131, 255, 0, 98, 255, 0, 65, 255,
    0, 32, 255, 0, 0, 255
};
static float comX, comY;

void psmm_plotinit();
void psmm_plotend();
void psmm_pscode();
void psmm_line();
void psmm_plot();
void psmm_setbkcolor();
void psmm_setcolor();
void psmm_setqbcolor();
void psmm_alloccolor();
void psmm_setline();
void psmm_setdash();
void psmm_setgrayshade();
void psmm_symbol();
void psmm_circle();
void psmm_arc();
void psmm_square();
void psmm_plus();
void psmm_times();
void psmm_rect();
void psmm_triangle();
void psmm_triangle1();
void psmm_triangle2();
void psmm_triangle3();
void psmm_polygon();
void psmm_imagefill();
void psmm_diamond();
void psmm_setfont();
void psmm_text();
void psmm_vector();
void psmm_translate();
void psmm_moveto();
void psmm_lineto();
void psmm_rlineto();

void psmm_plotinit(plotfile, xsize, ysize)
char *plotfile;
float xsize, ysize;
{
    double xoff = 0.1, yoff = 0.1;
    double xscl = 1.0, yscl = 1.0;
    double page_width = 8.3;
    int overlay = 0;
    int mode;
    int ncopies = 1;
    int dpi = 300;
    int unit = 1;		/* inch */
    int rgb[3];
    struct EPS *eps;
    double ratio;
    rgb[0] = rgb[1] = rgb[2] = 255;
    if (PS_COLOR) {
	rgb[0] = PS_R;
	rgb[1] = PS_G;
	rgb[2] = PS_B;
    }

    scale = 1. / 25.4;		/*   inch-> mm */

    ratio = ysize / xsize;
    if (ratio < 1.0)
	mode = 0;		/*  LandScape */
    else
	mode = 1;		/*  Portrait  */

    ps_plotinit(plotfile, overlay, mode, xoff, yoff, xscl, yscl, ncopies, dpi, unit, page_width, rgb, (struct EPS *) NULL);
    ps_setfont(0);		/* Helvetica */
    psmm_setgrayshade(-1);	/* white */
/*  ps_text(0.35,0.35,7,"TSEIS H.TSURUOKA",0.,1,1); */
    comX = 0.0;
    comY = 0.0;
}

void psmm_plotend(end)
int end;
{
    int lastpage = 1;
    ps_plotend(lastpage);
/*
  if (end){
    system(PS_PRINT); 
    printf("  PSLIB output [graph.ps] --> lpr\n");
  }
 */
}

void psmm_pscode(pscode)
int pscode;
{
    PS_COLOR = pscode;
}

void psmm_line(x1, y1, x2, y2)
float x1, y1, x2, y2;
{
    double x[2], y[2];
    int n = 2;
    int itype = 3;
    int iclose = 0;
    x[0] = x1 * scale;
    x[1] = x2 * scale;
    y[0] = y1 * scale;
    y[1] = y2 * scale;
    ps_line(x, y, n, itype, iclose);
}

void psmm_vector(x, y, direction, length, aw, hl, hw, outline)
float x, y, direction, length;
float aw, hl, hw;
int outline;
{
    double xtail, ytail, xtip, ytip;
    /*
       double tailwidth=0.01;
       double headlength=0.07;
       double headwidth=0.05;
     */
    double tailwidth;
    double headlength;
    double headwidth;
    double headshape = 0.05;
    int r, g, b;
    tailwidth = aw;
    headlength = hl;
    headwidth = hw;
    xtail = x * scale;
    ytail = y * scale;
    xtip = (x + length * (cos(3.14159 / 180. * direction))) * scale;
    ytip = (y + length * (sin(3.14159 / 180. * direction))) * scale;
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_vector(xtail, ytail, xtip, ytip, tailwidth, headlength, headwidth, headshape, r, g, b, outline);
}

void psmm_plot(x, y, n)
float x[], y[];
int n;
{
    double *gx, *gy;
    int itype = 3;
    int iclose = 0;
    int i;
    gx = (double *) malloc((unsigned) (n * sizeof(double)));
    gy = (double *) malloc((unsigned) (n * sizeof(double)));
    for (i = 0; i < n; i++) {
	*(gx + i) = x[i] * scale;
	*(gy + i) = y[i] * scale;
    }
    ps_line(gx, gy, n, itype, iclose);
    free((char *) gx);
    free((char *) gy);
}

void psmm_setbkcolor(r, g, b)
int r, g, b;
{
    PS_R = r;
    PS_G = g;
    PS_B = b;
}

void psmm_setcolor(r, g, b)
int r, g, b;
{
    ;
    if (PS_COLOR) {
	R = r;
	G = g;
	B = b;
	ps_setpaint(r, g, b);
    }
}

void psmm_setqbcolor(no)
int no;
{
    ;
    if (PS_COLOR) {
	R = RGB[no][0];
	G = RGB[no][1];
	B = RGB[no][2];
	ps_setpaint(R, G, B);
    }
}

void psmm_alloccolor(no, r, g, b)
int no, r, g, b;
{
    ;
    if (PS_COLOR) {
	RGB[no][0] = r;
	RGB[no][1] = g;
	RGB[no][2] = b;
    }
}

void psmm_setline(linewidth)
int linewidth;
{
    ps_setline(linewidth);
}

void psmm_setdash(dashtype)
int dashtype;
{
    static char *pattern[] = { "",
	"4 4",
	"10 10",
	"5 3 1 3",
	"5 5 1 1"
    };
    int offset;
    offset = 0;
    if (dashtype == 1)
	ps_setdash(pattern[0], offset);
    else if (dashtype == 2)
	ps_setdash(pattern[1], offset);
    else if (dashtype == 3)
	ps_setdash(pattern[2], offset);
    else if (dashtype == 4)
	ps_setdash(pattern[3], offset);
    else if (dashtype == 5)
	ps_setdash(pattern[4], offset);
    else
	ps_setdash(pattern[0], offset);
}

void psmm_setgrayshade(grayshade)
int grayshade;
{
    shade = grayshade;
}

void psmm_symbol(x, y, size, symboltype, outline)
float x, y, size;
int symboltype;
int outline;
{
    if (symboltype == 1)
	psmm_circle(x, y, size, outline);
    else if (symboltype == 2)
	psmm_square(x, y, size, outline);
    else if (symboltype == 3)
	psmm_plus(x, y, size, outline);
    else if (symboltype == 4)
	psmm_triangle(x, y, size, outline);
    else if (symboltype == 5)
	psmm_diamond(x, y, size, outline);
    else if (symboltype == 6)
	psmm_triangle1(x, y, size, outline);
    else if (symboltype == 7)
	psmm_triangle2(x, y, size, outline);
    else if (symboltype == 8)
	psmm_triangle3(x, y, size, outline);
    else if (symboltype == 9)
	psmm_times(x, y, size, outline);
}

void psmm_circle(x, y, size, outline)
float x, y, size;
int outline;
{
    double xx, yy;
    double radius;
    int r, g, b;
    xx = x * scale;
    yy = y * scale;
    radius = size * scale;
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_circle(xx, yy, radius, r, g, b, outline);
}

void psmm_arc(x, y, size, angle1, angle2, outline)
float x, y, size;
int angle1, angle2, outline;
{
    double xx, yy, width, height;
    double radius, az1, az2;
    int r, g, b;
    xx = x * scale;
    yy = y * scale;
    radius = size * scale;
    az1 = (double) angle1;
    az2 = (double) angle2;
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    if (outline == 0)
	ps_pie(xx, yy, radius, az1, az2, r, g, b, outline);
    else
	ps_arc(xx, yy, radius, az1, az2, 3);
}

void psmm_square(x, y, size, outline)
float x, y, size;
int outline;
{
    double xx, yy;
    double width;
    int r, g, b;
    xx = x * scale;
    yy = y * scale;
    width = size * scale;
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_square(xx, yy, width, r, g, b, outline);
}

void psmm_plus(x, y, size, outline)
float x, y, size;
int outline;
{
    double xx, yy;
    double width;
    xx = x * scale;
    yy = y * scale;
    width = size * scale;
    ps_cross(xx, yy, width);
}

void psmm_times(x, y, size, outline)
float x, y, size;
int outline;
{
    double xx[2], yy[2];
    int n = 2;
    int itype = 3;
    int iclose = 0;
    xx[0] = (x - 0.5 * size) * scale;
    xx[1] = (x + 0.5 * size) * scale;
    yy[0] = (y - 0.5 * size) * scale;
    yy[1] = (y + 0.5 * size) * scale;
    ps_line(xx, yy, n, itype, iclose);
    yy[0] = (y + 0.5 * size) * scale;
    yy[1] = (y - 0.5 * size) * scale;
    ps_line(xx, yy, n, itype, iclose);
}

void psmm_rect(x0, y0, wx, wy, outline)
float x0, y0;
float wx, wy;
int outline;
{
    double x1, y1, x2, y2;
    int r, g, b;
    x1 = x0 * scale;
    y1 = y0 * scale;
    x2 = (x0 + wx) * scale;
    y2 = (y0 + wy) * scale;
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_rect(x1, y1, x2, y2, r, g, b, outline);
}

void psmm_triangle(x, y, side, outline)
float x, y, side;
int outline;
{
    double xx, yy, size;
    int r, g, b;
    xx = x * scale;
    yy = y * scale;
    size = side * scale;
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_triangle(xx, yy, size, r, g, b, outline);
}

void psmm_triangle1(x, y, side, outline)
float x, y, side;
int outline;
{
    double px[3], py[3], size;
    int i, n = 3;
    int r, g, b;
    px[0] = x - 0.57736 * side;
    py[0] = y;
    px[1] = x + 0.28867 * side;
    py[1] = y - 0.5 * side;
    px[2] = px[1];
    py[2] = y + 0.5 * side;
    for (i = 0; i < 3; i++) {
	px[i] *= scale;
	py[i] *= scale;
    }
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_polygon(px, py, n, r, g, b, outline);
}

void psmm_triangle2(x, y, side, outline)
float x, y, side;
int outline;
{
    double px[3], py[3], size;
    int i, n = 3;
    int r, g, b;
    px[0] = x - 0.5 * side;
    py[0] = y + 0.28867 * side;
    px[1] = x;
    py[1] = y - 0.57736 * side;
    px[2] = x + 0.5 * side;
    py[2] = py[0];
    for (i = 0; i < 3; i++) {
	px[i] *= scale;
	py[i] *= scale;
    }
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_polygon(px, py, n, r, g, b, outline);
}

void psmm_triangle3(x, y, side, outline)
float x, y, side;
int outline;
{
    double px[3], py[3], size;
    int i, n = 3;
    int r, g, b;
    px[0] = x + 0.57736 * side;
    py[0] = y;
    px[1] = x - 0.28867 * side;
    py[1] = y - 0.5 * side;
    px[2] = px[1];
    py[2] = y + 0.5 * side;
    for (i = 0; i < 3; i++) {
	px[i] *= scale;
	py[i] *= scale;
    }
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_polygon(px, py, n, r, g, b, outline);
}

void psmm_polygon(x, y, n, outline)
float x[], y[];
int n;
int outline;
{
    double *xx, *yy;
    int r, g, b;
    int i;
    xx = (double *) malloc((unsigned) (n * sizeof(double)));
    yy = (double *) malloc((unsigned) (n * sizeof(double)));
    for (i = 0; i < n; i++) {
	*(xx + i) = x[i] * scale;
	*(yy + i) = y[i] * scale;
    }
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_polygon(xx, yy, n, r, g, b, outline);
    free((char *) xx);
    free((char *) yy);
}

void psmm_imagefill(x, y, n, image_no, invert, outline)
float x[], y[];
int n;
int image_no;
int invert;
int outline;
{
    double *xx, *yy;
    int i;
    xx = (double *) malloc((unsigned) (n * sizeof(double)));
    yy = (double *) malloc((unsigned) (n * sizeof(double)));
    for (i = 0; i < n; i++) {
	*(xx + i) = x[i] * scale;
	*(yy + i) = y[i] * scale;
    }
    ps_imagefill(xx, yy, n, image_no, "", invert, 0., outline);
    free((char *) xx);
    free((char *) yy);
}

void psmm_diamond(x, y, side, outline)
float x, y, side;
int outline;
{
    double xx, yy, size;
    int r, g, b;
    xx = x * scale;
    yy = y * scale;
    size = side * scale;
    r = g = b = shade;
    if ((outline == 0) && (shade == -1))
	r = g = b = 0;
    if (PS_COLOR) {
	r = R;
	g = G;
	b = B;
	if (outline == 1)
	    r = -1;
    }
    ps_diamond(xx, yy, size, r, g, b, outline);
}

void psmm_setfont(font_no)
int font_no;
{
    ps_setfont(font_no);
}

void psmm_text(x, y, size, text, angle, ntext, position)
float x, y, size;
char *text;
float angle;
int ntext;
int position;
{
    double xx, yy;
    char tmptext[512];
    int form = 0;
    int pointsize;
    int justify;
    xx = x * scale;
    yy = y * scale;
    pointsize = (int) (size * scale * 72.);
    if (position == 1)
	justify = 1;
    if (position == 2)
	justify = 2;
    if (position == 3)
	justify = 3;
    if (position == 4)
	justify = 5;
    if (position == 5)
	justify = 6;
    if (position == 6)
	justify = 7;
    if (position == 7)
	justify = 9;
    if (position == 8)
	justify = 10;
    if (position == 9)
	justify = 11;
    /*  text[ntext]='\0'; */
    strcpy(tmptext, text);
    tmptext[ntext] = '\0';
    ps_text(xx, yy, pointsize, tmptext, (double) angle, justify, form);
}

void psmm_translate(x, y)
float x, y;
{
    double gx, gy;
    gx = x * scale;
    gy = y * scale;
    ps_transrotate(gx, gy, 0.0);
}

void psmm_moveto(x, y)
float x, y;
{
    comX = x;
    comY = y;
}

void psmm_lineto(x, y)
float x, y;
{
    double gx[2], gy[2];
    int n = 2;
    int itype = 3;
    int iclose = 0;
    gx[0] = comX * scale;
    gx[1] = x * scale;
    gy[0] = comY * scale;
    gy[1] = y * scale;
    ps_line(gx, gy, n, itype, iclose);
    comX = x;
    comY = y;
}

void psmm_rlineto(dx, dy)
float dx, dy;
{
    double gx[2], gy[2];
    int n = 2;
    int itype = 3;
    int iclose = 0;
    gx[0] = comX * scale;
    gx[1] = (comX + dx) * scale;
    gy[0] = comY * scale;
    gy[1] = (comY + dy) * scale;
    ps_line(gx, gy, n, itype, iclose);
    comX += dx;
    comY += dy;
}
