#!/bin/sh
#
# xetas.sh
#
#./tseis2etas 2013 04 13 05 33  17.75 < awaji.ts > work.etas

# for EIC
#export ETAS_HOME=/usr/local/prog/etas

WINSCALE=0.8
BINWIDTH=8
#CATALOG=$ETAS_CATALOG

#while getopts c:s:b: OPT
while getopts cs:b:hm:r:t OPT
do
  case $OPT in
    "c" ) CONSOLE="TRUE" ;;
    "s" ) WINSCALE="$OPTARG" ;;
    "b" ) BINWIDTH="$OPTARG" ;;
    "h" ) HELP="TRUE" ;;
    "m" ) XMAG0="$OPTARG" ;;
    "r" ) REFTIME="$OPTARG" ;;
    "t" ) synTh="$OPTARG" ;;
  esac
done

if [ "$HELP" = "TRUE" ]; then
  cat <<EOM
Usage xetas.sh OPTION CATALOG
     OPTION -s winscale
            -b binwidth
            -m xmag0
            -c CONSOLE mode
            -r reference time yr mo dy hr mi sc (I4I2I2I2I2)
            -h help
     ARGVs  CATALOG
e.g. xetas.sh -s0.8 -h8 xlist.dat 
EOM
  exit 0
fi

shift $((OPTIND - 1))

OPTwish=""
if [ ! x${WINSCALE} = x ]; then
  OPTwish="$OPTwish -s$WINSCALE"
fi
if [ ! x${BINWIDTH} = x ]; then
  OPTwish="$OPTwish -b$BINWIDTH"
fi
if [ ! x${synTh} = x ]; then
  OPTwish="$OPTwish -t$synTh"
fi
OPTtseis2etas=""
if [ ! x${XMAG0} = x ]; then
  OPTtseis2etas="-m$XMAG0"
fi
if [ ! x${REFTIME} = x ]; then
  OPTtseis2etas="$OPTtseis2etas -r$REFTIME"
fi

#-if [ x$CATALOG = x ]; then
#-   export ETAS_CATALOG=xlist.dat
#-else
#- export ETAS_CATALOG=$CATALOG
#-fi

if [ x$1 = x ]; then
 if [ x${ETAS_CATALOG} = x ]; then
   export ETAS_CATALOG=xlist.dat
 fi
else
 export ETAS_CATALOG=$1
fi

echo "ETAS_CATALOG -> $ETAS_CATALOG"

if [ -f etas.lock ]; then
 \rm etas.lock
 echo "rm etas.lock"
fi

WC_XLIST=`wc -l $ETAS_CATALOG | awk '{print $1}'`
if [ "$WC_XLIST" -gt 17777 ]; then
	echo "$ETAS_CATALOG has 17777 records"
	exit
fi

trap 'echo "\nrm etas.lock"; \rm etas.lock; exit 1' 1 2 3 15

$ETAS_HOME/bin/tseis2etas $OPTtseis2etas < $ETAS_CATALOG > work.etas
$ETAS_HOME/bin/mk_etasopen.pl > etas.open
cat /dev/null > work.log
cat /dev/null > etas.log
if [ ! "$CONSOLE" = "TRUE" ]; then
   wish $ETAS_HOME/bin/xetastail.tcl work.log &
fi
$ETAS_HOME/bin/etasx >> work.log 2>&1
$ETAS_HOME/bin/retas >> work.log 2>&1
echo "*** cal END ***"
echo "*** cal END ***" >> work.log 2>&1
if [ ! "$CONSOLE" = "TRUE" ]; then
   $ETAS_HOME/bin/etasplot -s$WINSCALE &
   wish $ETAS_HOME/bin/xetas.tcl $OPTwish
fi
if [ "$CONSOLE" = "TRUE" ]; then
   mu=`head -4 etas.out | tail -1 | awk '{printf("%g",$1);}'`
   K=`head -4 etas.out | tail -1 | awk '{printf("%g",$2);}'`
   c=`head -4 etas.out | tail -1 | awk '{printf("%g",$3);}'`
   alpha=`head -4 etas.out | tail -1 | awk '{printf("%g",$4);}'`
   p=`head -4 etas.out | tail -1 | awk '{printf("%g",$5);}'`
   aic2=`head -6 etas.out | tail -1 | awk '{printf("%g",$1);}'`
   echo "mu= $mu"
   echo "K= $K"
   echo "c= $c"
   echo "alpha= $alpha"
   echo "p= $p"
   echo "aic/2= $aic2"
fi
