/*  PROGRAM xplot.c                                  */
/*  plot routine  for SCREEN                         */
/*        by H.Tsuruoka 8/9/1994                     */
/*  ADD xwin_rescale routine 9/02/1994               */
/*  ADD xwin_arc routine     9/02/1994               */
/*      xwin_resize routine  9/09/1994               */
/*      xwin_lines routine   9/12/1994               */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

#include <stdio.h>
#include <string.h>
#include <math.h>

#define CROSS_COLOR     "navy"

#define WIN_WIDTH_MM  400.	/* Width (mm) of window */
#define WIN_HEIGHT_MM 300.	/* Height (mm) of window */
#define WIN_BORDER_WIDTH 4	/* Width of border of window */
#define NCOLOR 	 64		/* Total no. of colors */
#define MAX_LINE_WIDTH 1600	/* Max. size of width of line (dots) */
#define MAX_WIN   4
#define MAX_LAYER   3

/*---------------> Public functions <---------------*/
void xwin_plotinit();		/* Open window */
void xwin_setbkcolor();		/* Set back color */
void xwin_pen();		/* Set color & line width of pen */
void xwin_setcolor();		/* Set color (R,G,B) */
void xwin_alloccolor();		/* Allocation color */
void xwin_setqbcolor();		/* Set color of allocate color */
void xwin_line();		/* Draw line */
void xwin_linenoflush();	/* Draw line */
void xwin_lines();		/* Draw lines */
void xwin_linetp();		/* Set line type */
void xwin_setline();		/* Set line width */
void xwin_setdash();		/* Set line type */
void xwin_point();		/* Draw point */
void xwin_circle();		/* Draw circle */
void xwin_squa();		/* Draw square */
void xwin_mouse();		/* Get mouse key & position */
void xwin_flush();		/* Flush buffer */
void xwin_clear();		/* Clear window[iwin] */
void xwin_cleararea();		/* Clear area H.Tsuru */
void xwin_plotend();		/* Close window[iwin] */
void xwin_scaletrn();
void xwin_text();		/* Draw Text */
void xwin_text2();		/* Draw Text */
void xwin_polygon();		/* Draw filled polygon */
void xwin_rescale();		/* Rescale Window */
void xwin_resize();		/* Resize Window */
void xwin_map();
void xwin_unmap();
void xwin_raise();		/* Raise Window       */
void xwin_arc();		/* Draw Arc           */
void xwin_bell();		/* ring the bell      */
void xwin_translate();		/* coordinate translate 98.04.24 */
void xwin_moveto();
void xwin_lineto();
void xwin_rlinto();
void xwin_setlayer();
void xwin_viewlayer();
void xwin_copylayer();

/*---------------> Private variables & functions <------------*/
static Display *display[MAX_WIN];
static Window window[MAX_WIN];
static Pixmap pixmap[MAX_WIN][MAX_LAYER];
static int layer[MAX_WIN];
static int screen[MAX_WIN];
static GC current_gc[MAX_WIN][MAX_LAYER];
static GC crosshair_gc[MAX_WIN];
static Cursor cross_cs[MAX_WIN], arrow_cs[MAX_WIN], crosshair_cs[MAX_WIN];
static XEvent event[MAX_WIN];
static float scale_x[MAX_WIN], scale_y[MAX_WIN], scale_1mm[MAX_WIN];
static int win_width[MAX_WIN], win_height[MAX_WIN], win_x0[MAX_WIN], win_y0[MAX_WIN];
static float win_wid_mm[MAX_WIN], win_hei_mm[MAX_WIN];
static unsigned long pixel[MAX_WIN][NCOLOR];
static int nfont[MAX_WIN];
static int bk_r[MAX_WIN], bk_g[MAX_WIN], bk_b[MAX_WIN];
static int fg_r[MAX_WIN], fg_g[MAX_WIN], fg_b[MAX_WIN];
static float orgX[MAX_WIN], orgY[MAX_WIN], comX[MAX_WIN], comY[MAX_WIN];

static unsigned long xwin_mycolor();
static unsigned long xwin_rgbcolor();
static void xwin_pixel_init();
static void xwin_coord_conv();
static void xwin_coord_inv();
static void xwin_scale();
static void xwin_crosshair_create();
static void xwin_crosshair_draw();

/************************************************************
 * xwin_plotinit: Open window and calculate scale (dots/mm)
 ************************************************************/
void xwin_plotinit(iwin, win_name, win_org_x, win_org_y, max_mm_x, max_mm_y, scale)
int iwin;
char *win_name;			/* Name of window[iwin]         (in) */
float max_mm_x, max_mm_y;	/* Max draw area (mm)   (in) */
float win_org_x, win_org_y, scale;
{
    XSizeHints hints;
    XSetWindowAttributes attrib;
    Cursor wait_cs;
    int border_color, back_color;
    int default_color, default_line;
    int i;
    int dargc = 0;
    char dargv[2][10];
    dargv[0][0] = 'x';		/* Dummy arguments */
    dargv[0][1] = '\0';
    nfont[iwin] = 0;

	/*-------------> Use default display[iwin] <-------------*/
    /*
     * if      (iwin == 0) {
     * display[iwin]=XopenDisplay(getenv("PLOT_WIN1")); } else if (iwin
     * == 1) { display[iwin]=XOpenDisplay(getenv("PLOT_WIN2")); } else if
     * (iwin == 2) { display[iwin]=XOpenDisplay(getenv("PLOT_WIN3")); }
     * else if (iwin == 3) {
     * display[iwin]=XOpenDisplay(getenv("PLOT_WIN4")); } else{
     * display[iwin] = XOpenDisplay(NULL); }
     */
    display[iwin] = XOpenDisplay(NULL);
    /* display[iwin] = XOpenDisplay(NULL); */
    if (display[iwin] == NULL) {
	printf("Cannot open display\n");
	printf("Check environment variable DISPLAY\n");
	exit(2);		/***** Abend *****/
    }
    screen[iwin] = DefaultScreen(display[iwin]);

    /*
     * if((scale < 0.5) | (scale > 1.5)) { scale=0.85; }
     */

	/*----------------> Calculate scale <--------------*/
    xwin_scale(iwin, win_org_x, win_org_y, max_mm_x, max_mm_y, scale);

	/*----------------> Open window[iwin] <--------------*/
    border_color = BlackPixel(display[iwin], screen[iwin]);
    back_color = WhitePixel(display[iwin], screen[iwin]);
    window[iwin] =
	XCreateSimpleWindow(display[iwin],
			    RootWindow(display[iwin], screen[iwin]),
			    win_x0[iwin], win_y0[iwin], win_width[iwin],
			    win_height[iwin], WIN_BORDER_WIDTH, border_color, back_color);
    for(i=0;i<MAX_LAYER;i++){
    pixmap[iwin][i] = XCreatePixmap(display[iwin],window[iwin],win_width[iwin],win_height[iwin],
                           DefaultDepth(display[iwin], 0));
    }


	/*-----------> Use backing store always <----------*/
/*
    attrib.backing_store = Always;
    XChangeWindowAttributes(display[iwin], window[iwin], CWBackingStore, &attrib);
 */

	/*-----------> Set properties of window[iwin] <------------*/
    hints.flags = USPosition | USSize | PMinSize | PMaxSize;
    /*
     * hints.min_width = 0.05*win_width[iwin]; hints.min_height
     * =0.05*win_height[iwin]; hints.max_width =
     * DisplayWidth(display[iwin], screen[iwin]); hints.max_height =
     * DisplayHeight(display[iwin], screen[iwin]);
     */
    hints.min_width = win_width[iwin];
    hints.min_height = win_height[iwin];
    hints.max_width = win_width[iwin];
    hints.max_height = win_height[iwin];
    XSetWMNormalHints(display[iwin], window[iwin], &hints);
    XStoreName(display[iwin], window[iwin], win_name);
    XSetIconName(display[iwin], window[iwin], win_name);

	/*--------> Setup default color & line attributes <-----------*/
    for(i=0;i<MAX_LAYER;i++){
    current_gc[iwin][i] = XCreateGC(display[iwin], pixmap[iwin][i], 0, 0);
    XSetGraphicsExposures(display[iwin],current_gc[iwin][i],False);
    }
    layer[iwin] = 0;

    xwin_pixel_init(iwin);
    for(i=0;i<MAX_LAYER;i++){
    xwin_setlayer(iwin,i);
    xwin_setbkcolor(iwin, 255, 255, 255);
    xwin_setcolor(iwin, 0, 0, 0);
    }
    orgX[iwin] = orgY[iwin] = 0.0;
    comX[iwin] = comY[iwin] = 0.0;

	/*----------> Create cursor <-----------*/
    cross_cs[iwin] = XCreateFontCursor(display[iwin], XC_crosshair);
    arrow_cs[iwin] = XCreateFontCursor(display[iwin], XC_arrow);
    xwin_crosshair_create(iwin);

	/*------------> Set event[iwin] mask <-------------------*/
    XSelectInput(display[iwin], window[iwin],
		 StructureNotifyMask | PointerMotionMask | ButtonPressMask | LeaveWindowMask | KeyPressMask | ExposureMask );

	/*------------> Show wait_cursor <----------------*/
    wait_cs = XCreateFontCursor(display[iwin], XC_watch);
    XDefineCursor(display[iwin], window[iwin], wait_cs);

	/*------------> Map window[iwin] <-----------------------*/
    XMapWindow(display[iwin], window[iwin]);

	/*------------> Wait until window[iwin] becomes active <---------*/
    while (1) {
	XNextEvent(display[iwin], &event[iwin]);
	if (event[iwin].type == MapNotify) {
	    break;
	}
		/*-----------> Use default cursor <--------------*/
	XUndefineCursor(display[iwin], window[iwin]);
    }

    XClearWindow(display[iwin], window[iwin]);
    XFlush(display[iwin]);
}

/*****************************************/
/* xwin_setbkcolor: Set background color */
/*****************************************/
void xwin_setbkcolor(iwin, r, g, b)
int iwin;
int r, g, b;			/* Color of background */
{
    Colormap cm;
    XColor c;
    cm = DefaultColormap(display[iwin], screen[iwin]);
    c.red = (r) * 257;
    c.green = (g) * 257;
    c.blue = (b) * 257;
    bk_r[iwin] = r;
    bk_g[iwin] = g;
    bk_b[iwin] = b;
    XAllocColor(display[iwin], cm, &c);
    XSetWindowBackground(display[iwin], window[iwin], c.pixel);
    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], c.pixel);
    XFillRectangle(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], 0, 0, win_width[iwin], win_height[iwin]);
    XClearWindow(display[iwin], window[iwin]);
    XFlush(display[iwin]);
    c.red = (fg_r[iwin]) * 257;
    c.green = (fg_g[iwin]) * 257;
    c.blue = (fg_b[iwin]) * 257;
    XAllocColor(display[iwin], cm, &c);
    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], c.pixel);
}

/************************************************************
 * xwin_pen: Set color & width of pen
 ************************************************************/
void xwin_pen(iwin, color, width)
int iwin;
int color, width;		/* Color no. & line width (in) */
{
	/*---------> Check argument <-------------------*/
    if (color >= NCOLOR)
	color = NCOLOR - 1;
    else if (color < 0)
	color = 0;

    if (width > MAX_LINE_WIDTH)
	width = MAX_LINE_WIDTH;
    else if (width <= 1)
	width = 0;		/* faster than wid=1 */

	/*---------> Set color & line width to current_gc[iwin] <-----*/
    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], pixel[iwin][color]);
    XSetLineAttributes(display[iwin], current_gc[iwin][layer[iwin]], width, LineSolid, CapRound, JoinRound);
}

/************************************************************
 * xwin_setcolor: Set color
 ************************************************************/
void xwin_setcolor(iwin, r, g, b)
int iwin;
int r, g, b;
{
    Colormap cm;
    static XColor c;
    cm = DefaultColormap(display[iwin], screen[iwin]);
    c.red = (r) * 257;
    c.green = (g) * 257;
    c.blue = (b) * 257;
    XAllocColor(display[iwin], cm, &c);
    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], c.pixel);
    /*
     * int no=20; xwin_alloccolor(iwin,no,r,g,b);
     * xwin_setqbcolor(iwin,no);
     */
    fg_r[iwin] = r;
    fg_g[iwin] = g;
    fg_b[iwin] = b;
}

/************************************************************
 * xwin_alloccolor: Allocate color
 ************************************************************/
void xwin_alloccolor(iwin, no, r, g, b)
int iwin;
int no;
int r, g, b;
{
    Colormap cm;
    XColor c;
    /* display[iwin]=XOpenDisplay(NULL); */
    cm = DefaultColormap(display[iwin], screen[iwin]);
    c.red = (r) * 257;
    c.green = (g) * 257;
    c.blue = (b) * 257;
    XAllocColor(display[iwin], cm, &c);
    pixel[iwin][no] = c.pixel;
}

/************************************************************
 * xwin_setqbcolor: Set color & width of pen
 ************************************************************/
void xwin_setqbcolor(iwin, color)
int iwin, color;
{
    if (color >= NCOLOR)
	color = NCOLOR - 1;
    else if (color < 0)
	color = 0;

	/*---------> Set color   <-----*/
    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], pixel[iwin][color]);
}

/************************************************************
 * xwin_linetp: Set line type
 ************************************************************/
void xwin_linetp(iwin, color, width, ityp)
int iwin;
int color, width;		/* Color no. & line width (in) */
int ityp;
{
    int col, wid, typ;
	/*---------> Check argument <-------------------*/

    col = color;
    if (col >= NCOLOR)
	col = NCOLOR - 1;
    else if (col < 0)
	col = 0;

    wid = width;
    if (wid > MAX_LINE_WIDTH)
	wid = MAX_LINE_WIDTH;
    else if (wid <= 1)
	wid = 1;		/* faster than wid=1 */

    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], pixel[iwin][col]);

    typ = ityp;
    if (typ == 1)
	XSetLineAttributes(display[iwin], current_gc[iwin][layer[iwin]], wid, LineSolid, CapRound, JoinRound);
    else if (typ == 2)
	XSetLineAttributes(display[iwin], current_gc[iwin][layer[iwin]], wid, LineOnOffDash, CapRound, JoinRound);
    else if (typ == 3)
	XSetLineAttributes(display[iwin], current_gc[iwin][layer[iwin]], wid, LineDoubleDash, CapRound, JoinRound);
    else
	XSetLineAttributes(display[iwin], current_gc[iwin][layer[iwin]], wid, LineSolid, CapRound, JoinRound);
}

/************************************************************
 * xwin_setline: Set line type
 ************************************************************/
void xwin_setline(iwin, linewidth)
int iwin;
int linewidth;
{
    XGCValues gv;
    gv.cap_style = CapRound;
    gv.join_style = JoinRound;
    gv.line_width = linewidth;
    XChangeGC(display[iwin], current_gc[iwin][layer[iwin]], GCLineWidth | GCCapStyle | GCJoinStyle, &gv);
}


/************************************************************
 * xwin_setdash: Set line type
 ************************************************************/
void xwin_setdash(iwin, itype)
int iwin;
int itype;
{
    XGCValues gv;
    gv.cap_style = CapRound;
    gv.join_style = JoinRound;
    if (itype == 1) {
	gv.line_style = LineSolid;
	XChangeGC(display[iwin], current_gc[iwin][layer[iwin]], GCLineStyle | GCCapStyle | GCJoinStyle, &gv);
    } else if (itype == 2) {
	gv.line_style = LineOnOffDash;
	XChangeGC(display[iwin], current_gc[iwin][layer[iwin]], GCLineStyle | GCCapStyle | GCJoinStyle, &gv);
    } else if (itype == 3) {
	gv.line_style = LineDoubleDash;
	XChangeGC(display[iwin], current_gc[iwin][layer[iwin]], GCLineStyle | GCCapStyle | GCJoinStyle, &gv);
    } else {
	gv.line_style = LineSolid;
	XChangeGC(display[iwin], current_gc[iwin][layer[iwin]], GCLineStyle | GCCapStyle | GCJoinStyle, &gv);
    }
}


/************************************************************
 * xwin_line: Draw line from (x1, y1) to (x2, y2) (unit:mm)
 ************************************************************/
void xwin_line(iwin, x1, y1, x2, y2)
int iwin;
float x1, y1, x2, y2;		/* 2 points (mm)        (in) */
{
    int idx1, idy1, idx2, idy2;
    float gx1, gy1, gx2, gy2;
    gx1 = orgX[iwin] + x1;
    gy1 = orgY[iwin] + y1;
    gx2 = orgX[iwin] + x2;
    gy2 = orgY[iwin] + y2;
    xwin_coord_conv(iwin, gx1, gy1, &idx1, &idy1);
    xwin_coord_conv(iwin, gx2, gy2, &idx2, &idy2);
    XDrawLine(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx1, idy1, idx2, idy2);
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
}

/************************************************************
 * xwin_linenoflush: Draw line from (x1, y1) to (x2, y2) (unit:mm)
 ************************************************************/
void xwin_linenoflush(iwin, x1, y1, x2, y2)
int iwin;
float x1, y1, x2, y2;		/* 2 points (mm)        (in) */
{
    int idx1, idy1, idx2, idy2;
    float gx1, gy1, gx2, gy2;
    gx1 = orgX[iwin] + x1;
    gy1 = orgY[iwin] + y1;
    gx2 = orgX[iwin] + x2;
    gy2 = orgY[iwin] + y2;
    xwin_coord_conv(iwin, gx1, gy1, &idx1, &idy1);
    xwin_coord_conv(iwin, gx2, gy2, &idx2, &idy2);
    XDrawLine(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx1, idy1, idx2, idy2);
/*
	XFlush(display[iwin]);
 */
}

/************************************************************
 * xwin_moveto: set (comX,comY)  (unit:mm)
 ************************************************************/
void xwin_moveto(iwin, x, y)
int iwin;
float x, y;			/* 2 points (mm)        (in) */
{
    comX[iwin] = x;
    comY[iwin] = y;
}


/************************************************************
 * xwin_lineto: Draw line from (orgX, orgY) to (x, y) (unit:mm)
 ************************************************************/
void xwin_lineto(iwin, x, y)
int iwin;
float x, y;			/* 2 points (mm)        (in) */
{
    int idx1, idy1, idx2, idy2;
    float gx1, gy1, gx2, gy2;
    gx1 = orgX[iwin] + comX[iwin];
    gy1 = orgY[iwin] + comY[iwin];
    gx2 = orgX[iwin] + x;
    gy2 = orgY[iwin] + y;
    xwin_coord_conv(iwin, gx1, gy1, &idx1, &idy1);
    xwin_coord_conv(iwin, gx2, gy2, &idx2, &idy2);
    XDrawLine(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx1, idy1, idx2, idy2);
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
    comX[iwin] = x;
    comY[iwin] = y;
}

/************************************************************
 * xwin_rlineto: Draw line from (orgX, orgY) to (orgX+dx, orgY+dy) (unit:mm)
 ************************************************************/
void xwin_rlineto(iwin, dx, dy)
int iwin;
float dx, dy;			/* 2 points (mm)        (in) */
{
    int idx1, idy1, idx2, idy2;
    float gx1, gy1, gx2, gy2;
    gx1 = orgX[iwin] + comX[iwin];
    gy1 = orgY[iwin] + comY[iwin];
    gx2 = orgX[iwin] + comX[iwin] + dx;
    gy2 = orgY[iwin] + comY[iwin] + dy;
    xwin_coord_conv(iwin, gx1, gy1, &idx1, &idy1);
    xwin_coord_conv(iwin, gx2, gy2, &idx2, &idy2);
    XDrawLine(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx1, idy1, idx2, idy2);
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
    comX[iwin] += dx;
    comY[iwin] += dy;
}

/************************************************************
 * xwin_lines: Draw line from (x1, y1) to (x2, y2) (unit:mm)
 ************************************************************/
void xwin_lines(iwin, x, y, n)
int iwin, n;
float x[], y[];			/* 2 points (mm)        (in) */
{
    int i, j, k, i1, i2, ix, iy, ii, nn;
    XPoint pt[2048];
    float gx, gy;
    if (n <= 2048)
	k = 1;
    else
	k = n / 2048 + 1;
    for (j = 0; j < k; j++) {
	i1 = 2047 * j;
	i2 = 2047 * (j + 1) + 1;
	nn = 2048;
	if (i2 > n) {
	    i2 = n;
	    nn = i2 - i1;
	}
	ii = 0;
	for (i = i1; i < i2; i++) {
	    gx = orgX[iwin] + x[i];
	    gy = orgY[iwin] + y[i];
	    xwin_coord_conv(iwin, gx, gy, &ix, &iy);
	    pt[ii].x = ix;
	    pt[ii].y = iy;
	    ii++;
	}
	XDrawLines(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], pt, nn, CoordModeOrigin);
    }
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
}

/************************************************************
 * xwin_point: Draw point at (x, y)  (unit:mm)
 ************************************************************/
void xwin_point(iwin, x, y)
int iwin;
float x, y;			/* Corrdinate of point (mm)     (in) */
{
    int idx, idy;
    float gx, gy;
    gx = orgX[iwin] + x;
    gy = orgY[iwin] + y;
    xwin_coord_conv(iwin, gx, gy, &idx, &idy);
    XDrawPoint(display[iwin], window[iwin], current_gc[iwin][layer[iwin]], idx, idy);
}

/************************************************************
 * xwin_text2: Draw Text at (x, y)
 ************************************************************/
void xwin_text2(iwin, x, y, size, string, length, ifontno, ixpos)
int iwin;
float x, y, size;		/* Corrdinate of point (mm)     (in) */
char *string;
int length, ifontno, ixpos;
{
    /* nfont[iwin] is move global variable Sep 1,1994 */
    static int fontsizemem[MAX_WIN][60];
    static Font fmem[MAX_WIN][60];
    static char fontnamemem[MAX_WIN][60][120];
    static fontnomem[MAX_WIN][60];
    Font f;
    static XFontStruct *fs[MAX_WIN][60];
    char fontname[120];
    int idx, idy, nlength, position;
    int fontsize, fontno, textwidth, pt;
    int i, equalfont, equalsize, equalname, j;
    float gx, gy;
    fontno = ifontno;
    /*
     * scale 1.0 ---> 1.0 scale 0.75 --> 1.0*0.75
     */
    fontsize = (int) (size / 25.4 * 72.27 * 1.05 * scale_1mm[iwin]);
    nlength = length;
    position = ixpos;
    pt = fontsize;
    switch (fontno) {
    case 0:
	sprintf(fontname, "-adobe-helvetica-medium-r-normal--%d-%d-75-75-p-0-iso8859-1", pt, 10 * pt);
	break;
    case 1:
	sprintf(fontname, "-adobe-helvetica-bold-r-normal--%d-%d-75-75-p-0-iso8859-1", pt, 10 * pt);
	break;
    case 2:
	sprintf(fontname, "-adobe-helvetica-medium-o-normal--%d-%d-75-75-p-0-iso8859-1", pt, 10 * pt);
	break;
    case 3:
	sprintf(fontname, "-adobe-helvetica-bold-o-normal--%d-%d-75-75-p-0-iso8859-1", pt, 10 * pt);
	break;
    default:
	sprintf(fontname, "-adobe-helvetica-medium-r-normal--%d-%d-75-75-p-0-iso8859-1", pt, 10 * pt);
    }
    /* printf("%s\n",fontname); */
    if (nfont[iwin] == 0) {
	fontsizemem[iwin][nfont[iwin]] = fontsize;
	fontnomem[iwin][nfont[iwin]] = fontno;
	strcpy(fontnamemem[iwin][nfont[iwin]], fontname);
	/* fontnamemem[iwin][nfont[iwin]]=fontname; */
	fmem[iwin][nfont[iwin]] = XLoadFont(display[iwin], fontname);
	fs[iwin][nfont[iwin]] = XLoadQueryFont(display[iwin], fontnamemem[iwin][nfont[iwin]]);
	j = 0;
	nfont[iwin]++;
    } else {
	for (i = 0; i < nfont[iwin]; i++) {
	    equalsize = (fontsizemem[iwin][i] == fontsize);
	    /* printf("size=%d\n",fontsize); */
	    equalname = (fontnomem[iwin][i] == fontno);
	    equalfont = (equalsize && equalname);

	    if (equalfont) {
		j = i;
		break;
	    }
	}
    }
    if (!equalfont) {
	fontsizemem[iwin][nfont[iwin]] = fontsize;
	strcpy(fontnamemem[iwin][nfont[iwin]], fontname);
	fontnomem[iwin][nfont[iwin]] = fontno;
	fmem[iwin][nfont[iwin]] = XLoadFont(display[iwin], fontname);
	fs[iwin][nfont[iwin]] = XLoadQueryFont(display[iwin], fontnamemem[iwin][nfont[iwin]]);
	j = nfont[iwin];
	nfont[iwin]++;
    }
    XSetFont(display[iwin], current_gc[iwin][layer[iwin]], fmem[iwin][j]);
    textwidth = XTextWidth(fs[iwin][j], string, nlength);
    gx = orgX[iwin] + x;
    gy = orgY[iwin] + y;
    xwin_coord_conv(iwin, gx, gy, &idx, &idy);
    switch (position) {
    case 1:
	idx = idx;
	break;
    case 2:
	idx = idx - textwidth / 2;
	break;
    case 3:
	idx = idx - textwidth;
	break;
    default:
	idx = idx;
    }
    XDrawString(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx, idy, string, nlength);
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
}


void xwin_text(iwin, xin, yin, size, text, angle, ntext, font, ipos)
int iwin;
float xin, yin, size, angle;
char *text;
int ntext;
int font;
int ipos;
{
    /*
     * 7       8        9 |~~~~~~~~~~~~~~~~| |4      +5      6|
     * |________________| 1       2        3
     */
    float sizemod, csize, radn, propotion;
    static int id[128], ic[1200];
    int idata[128];
    int textwidth, ixc, iyc, l, icndx, istrk, istyp;
    char s[1];
    int i, npen;
    float x, y, sine, cosin, sxloc, syloc, xct, yct, xchar;
    float x1, y1, xc, yc;
    int static load;
    if (load != 1) {
#include "font.h"
	load = 1;
    }
    xwin_setline(iwin, 1);
    xwin_setdash(iwin, 1);
    /* sizemod=0.600;  */
    sizemod = 0.600;
    csize = size / 21. * sizemod;	/* by Tsuru */
    radn = angle * 0.017453;
    propotion = 1.0;
    /* CALICULATION OF TEXT WIDTH  */
    for (i = 0; i < ntext; i++) {
	s[0] = text[i];
	idata[i] = s[0];
    }
    textwidth = 0.;
    for (i = 0; i < ntext; i++) {
	l = idata[i];		/* l is ascii code no */
	icndx = id[l + 1] - 1;	/* icndx is last data of text[i] */

	istrk = ic[icndx];
	istyp = istrk / 10000;
	ixc = ((istrk - (istyp * 10000)) / 100) - 10;
	iyc = istrk - (istyp * 10000) - ((ixc + 10) * 100) - 11;
	textwidth = textwidth + ixc;
    }

    xchar = (float) (textwidth) * sizemod / 21.;

    switch (ipos) {
    case 1:
	x = xin;
	y = yin;
	break;
    case 2:
	x = xin - cos(radn) * size * xchar / 2.;
	y = yin - sin(radn) * size * xchar / 2.;
	break;
    case 3:
	x = xin - cos(radn) * size * xchar;
	y = yin - sin(radn) * size * xchar;
	break;
    case 4:
	x = xin + sin(radn) * size / 2.;
	y = yin - cos(radn) * size / 2.;
	break;
    case 5:
	x = xin - cos(radn) * size * xchar / 2. + sin(radn) * size / 2.;
	y = yin - sin(radn) * size * xchar / 2. - cos(radn) * size / 2.;
	break;
    case 6:
	x = xin - cos(radn) * size * xchar + sin(radn) * size / 2.;
	y = yin - sin(radn) * size * xchar - cos(radn) * size / 2.;
	break;
    case 7:
	x = xin + sin(radn) * size;
	y = yin - cos(radn) * size;
	break;
    case 8:
	x = xin - cos(radn) * size * xchar / 2. + sin(radn) * size;
	y = yin - sin(radn) * size * xchar / 2. - cos(radn) * size;
	break;
    case 9:
	x = xin - cos(radn) * size * xchar + sin(radn) * size;
	y = yin - sin(radn) * size * xchar - cos(radn) * size;
	break;
    }
    sine = sin(radn);
    cosin = cos(radn);
    if (angle == 90.) {
	sine = 1.0;
	cosin = 0.;
    }
    x1 = x;
    y1 = y;
    sxloc = x;
    syloc = y;

    for (i = 0; i < ntext; i++) {
	l = idata[i];
	icndx = id[l];
	do {
	    istrk = ic[icndx];
	    istyp = istrk / 10000;
	    ixc = ((istrk - (istyp * 10000)) / 100) - 10;
	    iyc = istrk - (istyp * 10000) - ((ixc + 10) * 100) - 11;
	    xc = (float) (ixc) * propotion;
	    if (font == 2 || font == 3)
		xc = xc + (6. / 21.) * (float) iyc;	/* Oblique font */
	    yc = (float) iyc;
	    xc = xc * csize;
	    yc = yc * csize;
	    xct = xc * cosin - yc * sine;
	    yct = xc * sine + yc * cosin;
	    xct += sxloc;
	    yct += syloc;
	    /*
	     * stroke type 1 - move stroke type 2 - draw stroke
	     * type 3 - last move stroke type 0 - last draw
	     */
	    npen = 3;
	    if (istyp == 2 || istyp == 0)
		npen = 2;

	    if (npen == 2) {
		xwin_linenoflush(iwin, x1, y1, xct, yct);
		x1 = xct;
		y1 = yct;
	    } else if (npen == 3) {
		x1 = xct;
		y1 = yct;
	    }
	    icndx++;
	}
	while (istyp == 1 || istyp == 2);
	sxloc = xct;
	syloc = yct;
    }
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
}

/******************************************************************
 * xwin_circle: Draw circle at (x, y) with a radius of r (unit:mm)
 ******************************************************************/
void xwin_circle(iwin, x, y, radius, outline)
int iwin;
float x, y;			/* Corrdinate of left upper corner of point
				 * (mm)     (in) */
float radius;			/* Radius of circle  */
int outline;
{
    int idx, idy, idr, idd;
    float gx, gy;
    gx = orgX[iwin] + x;
    gy = orgY[iwin] + y;
    xwin_coord_conv(iwin, gx, gy, &idx, &idy);
    xwin_coord_conv(iwin, radius, radius, &idr, &idd);
    if (outline == 1)
	XDrawArc(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx, idy, idr, idr, 0 * 64, 360 * 64);
    else
	XFillArc(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx, idy, idr, idr, 0 * 64, 360 * 64);
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
}

/******************************************************************
 * xwin_arc: Draw Arc
 ******************************************************************/
void xwin_arc(iwin, x, y, width, height, angle1, angle2, outline)
int iwin;
float x, y;			/* Corrdinate of left upper corner of point
				 * (mm)     (in) */
float width, height;
int angle1, angle2;
int outline;
{
    int idx, idy, iwx, iwy, iwd, angle;
    float gx, gy;
    gx = orgX[iwin] + x;
    gy = orgY[iwin] + y;
    xwin_coord_conv(iwin, gx, gy, &idx, &idy);
    xwin_coord_conv(iwin, width, width, &iwx, &iwd);
    xwin_coord_conv(iwin, height, height, &iwy, &iwd);
    angle = angle2 - angle1;
    if (outline == 1)
	XDrawArc(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx, idy, iwx, iwy, angle1 * 64, angle * 64);
    else
	XFillArc(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx, idy, iwx, iwy, angle1 * 64, angle * 64);
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
}

/************************************************************
 * xwin_polygon: Draw filled circle at (x, y) with a radius of r (unit:mm)
 ************************************************************/
void xwin_polygon(iwin, px, py, n, outline)
int iwin;
float px[], py[];		/* Draw Point Value */
int n;				/* Number of Draw Point */
int outline;
{
    int i, ix, iy, i1, i2;
    XPoint p[2500];
    float x, y;
    for (i = 0; i < n; i++) {
	x = orgX[iwin] + px[i];
	y = orgY[iwin] + py[i];
	xwin_coord_conv(iwin, x, y, &ix, &iy);
	p[i].x = ix;
	p[i].y = iy;
    }
    if (outline == 1) {
	x = orgX[iwin] + px[0];
	y = orgY[iwin] + py[0];
	xwin_coord_conv(iwin, x, y, &ix, &iy);
	p[n].x = ix;
	p[n].y = iy;
	XDrawLines(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], p, n + 1, CoordModeOrigin);
	/*
	 * for(i=0;i<n;i++){ i1=i;    i2=i+1; if(i2==n) i2=0;
	 * XDrawLine(display[iwin],window[iwin],current_gc[iwin],p[i1]
	 * .x,p[i1].y,p[i2].x,p[i2].y); }
	 */
    } else
	XFillPolygon(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], p, n, Convex, CoordModeOrigin);
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
}


/************************************************************
 * xwin_squa: Draw square at (x, y) with a length of rx and ry (unit:mm)
 ************************************************************/
void xwin_squa(iwin, x, y, r, outline)
int iwin;
float x, y;			/* Corrdinate of left corner of point (mm)
				 * (in) */
float r;			/* width */
int outline;
{
    int idx, idy, irx, iry;
    float gx, gy;
    gx = orgX[iwin] + x;
    gy = orgY[iwin] + y;
    xwin_coord_conv(iwin, gx, gy, &idx, &idy);
    xwin_coord_conv(iwin, r, r, &irx, &iry);
    if (outline == 1)
	XDrawRectangle(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx, idy, irx, irx);
    else
	XFillRectangle(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], idx, idy, irx, irx);
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
}

/************************************************************
 * xwin_mouse: Get position of mouse cursor and key
 ************************************************************/
void xwin_mouse(iwin, itype, key, x, y)
int iwin;
int itype;			/* Cursor type (0:crosshair,1:cross,2:arrow)
				 * (in) */
int *key;			/* Mouse key (1, 2, 3)                  (out) */
float *x, *y;			/* Cursor postion (mm) from lower left  (out) */
{
    Window root, child;
    int rx, ry, wx, wy;
    unsigned int mask;
    int xbef, ybef;
/*      append by tsuru  */
    XSync(display[iwin], True);
/*
    xwin_raise(iwin);
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
    switch (itype) {
    case 0:
	XDefineCursor(display[iwin], window[iwin], crosshair_cs[iwin]);
	xbef = 0;
	ybef = 0;
	XQueryPointer(display[iwin], window[iwin], &root, &child, &rx, &ry, &wx, &wy, &mask);
	if (wx >= 0 && wx <= win_width[iwin] && wy >= 0 && wy <= win_height[iwin]) {
	    xwin_crosshair_draw(iwin, wx, wy);
	    xbef = wx;
	    ybef = wy;
	}
	break;
    case 1:
	XDefineCursor(display[iwin], window[iwin], cross_cs[iwin]);
	break;
    default:
	XDefineCursor(display[iwin], window[iwin], arrow_cs[iwin]);
	break;
    }
    while (1) {
	XSelectInput(display[iwin], window[iwin], StructureNotifyMask | PointerMotionMask | ButtonPressMask | LeaveWindowMask | ExposureMask);

	XNextEvent(display[iwin], &event[iwin]);
	switch (event[iwin].type) {
        case Expose:
            if (event[iwin].xexpose.count == 0 ) {
            XCopyArea(display[iwin],pixmap[iwin][layer[iwin]],window[iwin],current_gc[iwin][layer[iwin]],0,0,win_width[iwin],win_height[iwin],0,0);
            XFlush(display[iwin]);
/*
            printf("w h %d %d\n",win_width[iwin],win_height[iwin]);
 */
            }
            break;
	case LeaveNotify:
	    if (itype == 0) {
		xwin_crosshair_draw(iwin, xbef, ybef);
		xbef = -1.;
		ybef = -1.;
	    }
	    break;
	case MotionNotify:
	    if (itype == 0) {
		xwin_crosshair_draw(iwin, xbef, ybef);
		xwin_crosshair_draw(iwin, event[iwin].xmotion.x, event[iwin].xmotion.y);
		xbef = event[iwin].xmotion.x;
		ybef = event[iwin].xmotion.y;
	    }
	    break;
	case ButtonPress:
	    xwin_coord_inv(iwin, event[iwin].xbutton.x, event[iwin].xbutton.y, x, y);
	    *x = *x - orgX[iwin];
	    *y = *y - orgY[iwin];
	    /* XBell(display[iwin],100); */
	    switch (event[iwin].xbutton.button) {
	    case Button1:
		*key = 1;
		break;
	    case Button2:
		*key = 2;
		break;
	    case Button3:
		*key = 3;
		break;
	    }
	    if (itype == 0)
		xwin_crosshair_draw(iwin, xbef, ybef);	/* Erase crosshair */
	    XUndefineCursor(display[iwin], window[iwin]);
	    XFlush(display[iwin]);
	    return;
	}
    }
}

/************************************************************
 * xwin_flush: Flush buffer
 ************************************************************/
/*
void xwin_flush(iwin)
int iwin;
{
    XFlush(display[iwin]);
}
 */

/************************************************************
 * xwin_clear: Clear window
 ************************************************************/
void xwin_clear(iwin)
int iwin;
{
    Colormap cm;
    XColor c;
    cm = DefaultColormap(display[iwin], screen[iwin]);
    c.red = (bk_r[iwin]) * 257;
    c.green = (bk_g[iwin]) * 257;
    c.blue = (bk_b[iwin]) * 257;
    XAllocColor(display[iwin], cm, &c);
    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], c.pixel);
    XFillRectangle(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], 0, 0, win_width[iwin], win_height[iwin]);
/*
    XClearWindow(display[iwin], window[iwin]);
 */
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
    c.red = (fg_r[iwin]) * 257;
    c.green = (fg_g[iwin]) * 257;
    c.blue = (fg_b[iwin]) * 257;
    XAllocColor(display[iwin], cm, &c);
    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], c.pixel);
}

/***********************************************************
 * xwin_cleararea: Clear window[iwin] area
 *                        94/2/20 H.Tsuru
 ***********************************************************/
void xwin_cleararea(iwin, x1, y1, x2, y2)
int iwin;
float x1, y1, x2, y2;
{
    int ix1, iy1, ix2, iy2, iwx, iwy;
    /* unsigned int iwx,iwy; */
    Bool exposures;
    float gx1, gy1, gx2, gy2;
    Colormap cm;
    XColor c;
    gx1 = orgX[iwin] + x1;
    gy1 = orgY[iwin] + y1;
    gx2 = orgX[iwin] + x2;
    gy2 = orgY[iwin] + y2;
    xwin_coord_conv(iwin, gx1, gy1, &ix1, &iy1);
    xwin_coord_conv(iwin, gx2, gy2, &ix2, &iy2);
    iwx = ix2 - ix1;
    iwy = iy1 - iy2;
    /*
     * printf("iwx=%d iwy=%d x1=%d y1=%d x2=%d
     * y2=%d\n",iwx,iwy,ix1,iy1,ix2,iy2);
     */
    cm = DefaultColormap(display[iwin], screen[iwin]);
    c.red = (bk_r[iwin]) * 257;
    c.green = (bk_g[iwin]) * 257;
    c.blue = (bk_b[iwin]) * 257;
    XAllocColor(display[iwin], cm, &c);
    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], c.pixel);
    XFillRectangle(display[iwin], pixmap[iwin][layer[iwin]], current_gc[iwin][layer[iwin]], ix1, iy2, iwx, iwy);
/*
    XClearArea(display[iwin], window[iwin], ix1, iy2, iwx, iwy, False);
 */
/*
    XCopyArea(display[iwin],pixmap[iwin],window[iwin],current_gc[iwin],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
 */
    c.red = (fg_r[iwin]) * 257;
    c.green = (fg_g[iwin]) * 257;
    c.blue = (fg_b[iwin]) * 257;
    XAllocColor(display[iwin], cm, &c);
    XSetForeground(display[iwin], current_gc[iwin][layer[iwin]], c.pixel);
}

/************************************************************
 * xwin_plotend: Close window[iwin]
 ************************************************************/
void xwin_plotend(iwin)
int iwin;
{
    nfont[iwin] = 0;
    XCloseDisplay(display[iwin]);
}

/************************************************************
 * xwin_map: Map window[iwin]
 ************************************************************/
void xwin_map(iwin)
int iwin;
{
    XMapWindow(display[iwin], window[iwin]);
    XFlush(display[iwin]);
}

/************************************************************
 * xwin_unmap: Close window[iwin]
 ************************************************************/
void xwin_unmap(iwin)
int iwin;
{
    XUnmapWindow(display[iwin], window[iwin]);
    XFlush(display[iwin]);
}

/************************************************************
 * xwin_raise: Raise window[iwin]
 ************************************************************/
void xwin_raise(iwin)
int iwin;
{
    XRaiseWindow(display[iwin], window[iwin]);
    XFlush(display[iwin]);
}

/************************************************************
 * xwin_rescale: rescale window[iwin]
 ************************************************************/
void xwin_rescale(iwin, rescale)
int iwin;
int rescale;
{
    nfont[iwin] = 0;
    if (rescale > 0) {
	scale_x[iwin] = 1.2 * scale_x[iwin];
	scale_y[iwin] = 1.2 * scale_y[iwin];
	scale_1mm[iwin] = 1.2 * scale_1mm[iwin];
	win_width[iwin] = 1.2 * win_width[iwin];
	win_height[iwin] = 1.2 * win_height[iwin];
    }
    if (rescale < 0) {
	scale_x[iwin] = 0.85 * scale_x[iwin];
	scale_y[iwin] = 0.85 * scale_y[iwin];
	scale_1mm[iwin] = 0.85 * scale_1mm[iwin];
	win_width[iwin] = 0.85 * win_width[iwin];
	win_height[iwin] = 0.85 * win_height[iwin];
    }
    XResizeWindow(display[iwin], window[iwin], win_width[iwin], win_height[iwin]);
    XFlush(display[iwin]);
    sleep(1);
}

/************************************************************
 * xwin_resize: resize window[iwin]
 ************************************************************/
void xwin_resize(iwin, width, height)
int iwin;
float width, height;
{
    nfont[iwin] = 0;
    win_width[iwin] = scale_x[iwin] * width;
    win_height[iwin] = scale_y[iwin] * height;
    XResizeWindow(display[iwin], window[iwin], win_width[iwin], win_height[iwin]);
    XFlush(display[iwin]);
    sleep(1);
}

/************************************************************
 * xwin_bell: ring the bell
 *   -100<percent<100
 ************************************************************/
void xwin_bell(iwin, percent)
int iwin;
int percent;
{
    XBell(display[iwin], percent);
    XFlush(display[iwin]);
}

/************************************************************
 * xwin_translate: coordinate translate
 * 98.04.24
 ************************************************************/
void xwin_translate(iwin, x, y)
int iwin;
float x, y;
{
    orgX[iwin] = orgX[iwin] + x;
    orgY[iwin] = orgY[iwin] + y;
}

/************************************************************
 * xwin_flush
 * 04.05.10
 ************************************************************/
void xwin_flush(iwin)
int iwin;
{
    XCopyArea(display[iwin],pixmap[iwin][layer[iwin]],window[iwin],current_gc[iwin][layer[iwin]],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
}

/************************************************************
 * xwin_viewlayer
 * 04.05.10
 ************************************************************/
void xwin_viewlayer(iwin,layer)
int iwin,layer;
{
    XCopyArea(display[iwin],pixmap[iwin][layer],window[iwin],current_gc[iwin][layer],0,0,win_width[iwin],win_height[iwin],0,0);
    XFlush(display[iwin]);
}

/************************************************************
 * xwin_copylayer
 * 04.05.10
 ************************************************************/
void xwin_copylayer(iwin,layer1,layer2)
int iwin,layer1,layer2;
{
    XCopyArea(display[iwin],pixmap[iwin][layer1],pixmap[iwin][layer2],current_gc[iwin][layer1],0,0,win_width[iwin],win_height[iwin],0,0);
}

/*
 * ####################################################### #	Private
 * functions				#
 * #######################################################
 */
/*
 * ============================================================ xwin_mycolor:
 * Get pixel no. from color name
 * ============================================================
 */
static unsigned long xwin_mycolor(dsp, scr, color_name)
Display *dsp;			/* Display      (in) */
int scr;			/* Screen[iwin] no.   (in) */
char *color_name;		/* Name of color(in) */
{
    Colormap cmap;
    XColor c0, c1;
    cmap = DefaultColormap(dsp, scr);
    XAllocNamedColor(dsp, cmap, color_name, &c1, &c0);
    return (c1.pixel);
}

/*
 * ============================================================
 * xwin_rgbcolor: Set pixel no. from (R,G,B) [0-255]
 * ============================================================
 */
static unsigned long xwin_rgbcolor(dsp, scr, no, r, g, b)
Display *dsp;
int scr;
int no;
int r, g, b;
{
    Colormap cm;
    XColor c;
    cm = DefaultColormap(dsp, scr);
    c.red = (r) * 257;
    c.green = (g) * 257;
    c.blue = (b) * 257;
    XAllocColor(dsp, cm, &c);
    return (c.pixel);
}

/************************************************************
 * xwin_pixel_init: init the pixel table
 ************************************************************/
static void xwin_pixel_init(iwin)
int iwin;
{
    pixel[iwin][0] = BlackPixel(display[iwin], screen[iwin]);
    pixel[iwin][1] = xwin_mycolor(display[iwin], screen[iwin], "blue");
    pixel[iwin][2] = xwin_mycolor(display[iwin], screen[iwin], "green");
    pixel[iwin][3] = xwin_mycolor(display[iwin], screen[iwin], "cyan");
    pixel[iwin][4] = xwin_mycolor(display[iwin], screen[iwin], "red");
    pixel[iwin][5] = xwin_mycolor(display[iwin], screen[iwin], "magenta");
    pixel[iwin][6] = xwin_mycolor(display[iwin], screen[iwin], "yellow");
    pixel[iwin][7] = WhitePixel(display[iwin], screen[iwin]);

    pixel[iwin][8] = xwin_mycolor(display[iwin], screen[iwin], "aquamarine");
    pixel[iwin][9] = xwin_mycolor(display[iwin], screen[iwin], "blue violet");
    pixel[iwin][10] = xwin_mycolor(display[iwin], screen[iwin], "brown");
    pixel[iwin][11] = xwin_mycolor(display[iwin], screen[iwin], "cadet blue");
    pixel[iwin][12] = xwin_mycolor(display[iwin], screen[iwin], "coral");
    pixel[iwin][13] = xwin_mycolor(display[iwin], screen[iwin], "cyan");
    pixel[iwin][14] = xwin_mycolor(display[iwin], screen[iwin], "dark green");
    pixel[iwin][15] = xwin_mycolor(display[iwin], screen[iwin], "dark olive green");
    pixel[iwin][16] = xwin_mycolor(display[iwin], screen[iwin], "gray");
    pixel[iwin][17] = xwin_mycolor(display[iwin], screen[iwin], "orange");
    pixel[iwin][18] = xwin_mycolor(display[iwin], screen[iwin], "sea green");
    pixel[iwin][19] = xwin_mycolor(display[iwin], screen[iwin], "navy");
    pixel[iwin][20] = WhitePixel(display[iwin], screen[iwin]);

    /* Rainbow scale RED to BLUE  */
    /* RED[32]-BLUE[63]           */

    pixel[iwin][32] = xwin_rgbcolor(display[iwin], screen[iwin], 32, 255, 0, 0);
    pixel[iwin][33] = xwin_rgbcolor(display[iwin], screen[iwin], 33, 255, 32, 0);
    pixel[iwin][34] = xwin_rgbcolor(display[iwin], screen[iwin], 34, 255, 65, 0);
    pixel[iwin][35] = xwin_rgbcolor(display[iwin], screen[iwin], 35, 255, 98, 0);
    pixel[iwin][36] = xwin_rgbcolor(display[iwin], screen[iwin], 36, 255, 131, 0);
    pixel[iwin][37] = xwin_rgbcolor(display[iwin], screen[iwin], 37, 255, 164, 0);
    pixel[iwin][38] = xwin_rgbcolor(display[iwin], screen[iwin], 38, 255, 197, 0);
    pixel[iwin][39] = xwin_rgbcolor(display[iwin], screen[iwin], 39, 255, 230, 0);
    pixel[iwin][40] = xwin_rgbcolor(display[iwin], screen[iwin], 40, 246, 255, 0);
    pixel[iwin][41] = xwin_rgbcolor(display[iwin], screen[iwin], 41, 213, 255, 0);
    pixel[iwin][42] = xwin_rgbcolor(display[iwin], screen[iwin], 42, 180, 255, 0);
    pixel[iwin][43] = xwin_rgbcolor(display[iwin], screen[iwin], 43, 148, 255, 0);
    pixel[iwin][44] = xwin_rgbcolor(display[iwin], screen[iwin], 44, 115, 255, 0);
    pixel[iwin][45] = xwin_rgbcolor(display[iwin], screen[iwin], 45, 82, 255, 0);
    pixel[iwin][46] = xwin_rgbcolor(display[iwin], screen[iwin], 46, 49, 255, 0);
    pixel[iwin][47] = xwin_rgbcolor(display[iwin], screen[iwin], 47, 16, 255, 0);
    pixel[iwin][48] = xwin_rgbcolor(display[iwin], screen[iwin], 48, 0, 255, 16);
    pixel[iwin][49] = xwin_rgbcolor(display[iwin], screen[iwin], 49, 0, 255, 49);
    pixel[iwin][50] = xwin_rgbcolor(display[iwin], screen[iwin], 50, 0, 255, 82);
    pixel[iwin][51] = xwin_rgbcolor(display[iwin], screen[iwin], 51, 0, 255, 115);
    pixel[iwin][52] = xwin_rgbcolor(display[iwin], screen[iwin], 52, 0, 255, 148);
    pixel[iwin][53] = xwin_rgbcolor(display[iwin], screen[iwin], 53, 0, 255, 180);
    pixel[iwin][54] = xwin_rgbcolor(display[iwin], screen[iwin], 54, 0, 255, 213);
    pixel[iwin][55] = xwin_rgbcolor(display[iwin], screen[iwin], 55, 0, 255, 246);
    pixel[iwin][56] = xwin_rgbcolor(display[iwin], screen[iwin], 56, 0, 230, 255);
    pixel[iwin][57] = xwin_rgbcolor(display[iwin], screen[iwin], 57, 0, 197, 255);
    pixel[iwin][58] = xwin_rgbcolor(display[iwin], screen[iwin], 58, 0, 164, 255);
    pixel[iwin][59] = xwin_rgbcolor(display[iwin], screen[iwin], 59, 0, 131, 255);
    pixel[iwin][60] = xwin_rgbcolor(display[iwin], screen[iwin], 60, 0, 98, 255);
    pixel[iwin][61] = xwin_rgbcolor(display[iwin], screen[iwin], 61, 0, 65, 255);
    pixel[iwin][62] = xwin_rgbcolor(display[iwin], screen[iwin], 62, 0, 32, 255);
    pixel[iwin][63] = xwin_rgbcolor(display[iwin], screen[iwin], 63, 0, 0, 255);

}

/*
 * ============================================================
 * xwin_coord_conv: Convert coordinate system from (mm) to (dots)
 * ============================================================
 */
static void xwin_coord_conv(iwin, xorg, yorg, xdot, ydot)
int iwin;
float xorg, yorg;		/* Original coordinate (mm) origin is
				 * lower left */
int *xdot, *ydot;		/* Converted coordinate (dots) origin
				 * is upper left */
{
    /*
     * xdot = (int) ( xorg * scale_x[iwin] + 0.5); ydot = (int) (
     * (win_hei_mm[iwin] - yorg) * scale_y[iwin] + 0.5);
     */
    *xdot = (int) (xorg * scale_x[iwin]);
    *ydot = (int) ((win_height[iwin] / scale_y[iwin] - yorg) * scale_y[iwin]);
}

/*
 * ============================================================
 * xwin_coord_inv: Invert coordinate system from (dots) to (mm)
 * ============================================================
 */
static void xwin_coord_inv(iwin, xdot, ydot, xinv, yinv)
int iwin;
int xdot, ydot;			/* Original coordinate (dots) origin
				 * is upper left */
float *xinv, *yinv;		/* Inverted coordinate (mm) origin is
				 * lower left */
{
    *xinv = (float) xdot / scale_x[iwin];
    /*
     * yinv = win_hei_mm[iwin] - (float) ydot / scale_y[iwin];
     */
    *yinv = win_height[iwin] / scale_y[iwin] - (float) ydot / scale_y[iwin];
}

/*
 * ============================================================ xwin_scale:
 * Calculte convert scale (dots/mm) win_width, win_height, scale_x, scale_y,
 * win_wid_mm, win_hei_mm  will be calculated
 * ============================================================
 */
static void xwin_scale(iwin, win_org_x, win_org_y, maxmm_x, maxmm_y, scale)
int iwin;
float maxmm_x, maxmm_y;		/* Max. size(mm) of
				 * window[iwin] given by user
				 * (in) */
float win_org_x, win_org_y, scale;
{
    int disp_x_mm, disp_y_mm, disp_x_dot, disp_y_dot;
    float reduce_x, reduce_y, reduce;
    float disp_wid_mm, disp_hei_mm;

	/*-----> Default size = WIN_WIDTH_MM x WIN_HEIGHT_MM <----*/

    if (maxmm_x < WIN_WIDTH_MM) {
	win_wid_mm[iwin] = maxmm_x;
    } else {
	win_wid_mm[iwin] = WIN_WIDTH_MM;
    }

    if (maxmm_y < WIN_HEIGHT_MM) {
	win_hei_mm[iwin] = maxmm_y;
    } else {
	win_hei_mm[iwin] = WIN_HEIGHT_MM;
    }

	/*---------------> Get display[iwin] information <-----------------*/
    disp_x_mm = DisplayWidthMM(display[iwin], screen[iwin]);
    disp_y_mm = DisplayHeightMM(display[iwin], screen[iwin]);

    disp_x_dot = DisplayWidth(display[iwin], screen[iwin]);
    disp_y_dot = DisplayHeight(display[iwin], screen[iwin]);

    scale_x[iwin] = (float) disp_x_dot / (float) disp_x_mm;
    scale_y[iwin] = (float) disp_y_dot / (float) disp_y_mm;
    scale_1mm[iwin] = scale;
    scale_x[iwin] *= scale_1mm[iwin];
    scale_y[iwin] *= scale_1mm[iwin];

    disp_wid_mm = (float) disp_x_mm - 5.;
    disp_hei_mm = (float) disp_y_mm - 10.;

	/*-----> Reduce scale if display[iwin] is too small <----*/
    reduce_x = 1.;
    reduce_y = 1.;
    if (disp_wid_mm < win_wid_mm[iwin] * scale_1mm[iwin]) {
	reduce_x = disp_wid_mm / (win_wid_mm[iwin] * scale_1mm[iwin]);
	printf("Max X (%f mm) is too large\n", win_wid_mm[iwin]);
	printf("It will be reduced to %f mm\n", disp_wid_mm / scale_1mm[iwin]);
    }
    if (disp_hei_mm < win_hei_mm[iwin] * scale_1mm[iwin]) {
	reduce_y = disp_hei_mm / (win_hei_mm[iwin] * scale_1mm[iwin]);
	printf("Max Y (%f mm) is too large\n", win_hei_mm[iwin]);
	printf("It will be reduced to %f mm\n", disp_hei_mm / scale_1mm[iwin]);
    }
    if (reduce_x < reduce_y) {
	reduce = reduce_x;
    } else {
	reduce = reduce_y;
    }

    scale_x[iwin] *= reduce;
    scale_y[iwin] *= reduce;
    scale_1mm[iwin] *= reduce;

	/*--------> Calculate window[iwin] width & heights (dots) <-------*/
    win_width[iwin] = win_wid_mm[iwin] * scale_x[iwin];
    win_height[iwin] = win_hei_mm[iwin] * scale_y[iwin];
    win_x0[iwin] = win_org_x * scale_x[iwin] / scale_1mm[iwin];
    win_y0[iwin] = win_org_y * scale_y[iwin] / scale_1mm[iwin];

}

/*
 * ============================================================
 * xwin_crosshair_create: Create crosshair cursor Information of crosshair
 * will be stored in crosshair_gc
 * ============================================================
 */
static void xwin_crosshair_create(iwin)
int iwin;
{
    XColor c1, c2, cc;
    Colormap cm;
    Font f1;
    unsigned long black, crosshair_color;
    cm = DefaultColormap(display[iwin], screen[iwin]);
    XAllocNamedColor(display[iwin], cm, CROSS_COLOR, &c1, &cc);
    c2.red = bk_r[iwin] * 257;
    c2.green = bk_g[iwin] * 257;
    c2.blue = bk_b[iwin] * 257;
    XAllocColor(display[iwin], cm, &c2);
/*
    f1 = XLoadFont(display[iwin], "r14");
 */
    f1 = XLoadFont(display[iwin], "fixed");
    crosshair_cs[iwin] = XCreateGlyphCursor(display[iwin], f1, f1, ' ', ' ', &c1, &c2);
    crosshair_gc[iwin] = XCreateGC(display[iwin], window[iwin], 0, 0);
    black = BlackPixel(display[iwin], screen[iwin]);
	/*---------------------------------*/
    /* SET CROSSHAIR_COLOR      */
	/*---------------------------------*/
    crosshair_color = c2.pixel ^ c1.pixel;
    /* crosshair_color = c1.pixel ; */
    XSetFunction(display[iwin], crosshair_gc[iwin], GXxor);
    XSetForeground(display[iwin], crosshair_gc[iwin], crosshair_color);
    XSetLineAttributes(display[iwin], crosshair_gc[iwin], 1, LineSolid, CapButt, JoinMiter);
}

/*
 * ============================================================
 * xwin_crosshair_draw: Draw crosshair cursor
 * ============================================================
 */
static void xwin_crosshair_draw(iwin, x, y)
int iwin;
int x, y;			/* Position of cursor (dots)    (in) */
{
    XDrawLine(display[iwin], window[iwin], crosshair_gc[iwin], x, 0, x, win_height[iwin]);
    XDrawLine(display[iwin], window[iwin], crosshair_gc[iwin], 0, y, win_width[iwin], y);
}

/*
 * ================================================================
 * xwin_scaletrn(iwin,x,y)
 * ==============================================================
 */
void xwin_scaletrn(iwin, sx, sy)
int iwin;
float *sx, *sy;
{
    *sx = scale_x[iwin];
    *sy = scale_y[iwin];
}

/************************************************************
 * xwin_setlayer: Set layer
 ************************************************************/
void xwin_setlayer(iwin, nlayer)
int iwin;
int nlayer;
{
    layer[iwin] = nlayer;
}
